@@ -86,7 +86,7 @@
   {
     FT_Error   error;
     FT_Memory  memory = stream->memory;
-    FT_UShort  n, count;
+    FT_UShort  n, m, count;
     FT_ULong   cur_offset, new_offset, base_offset;
 
     TTO_LangSysRecord*  lsr;
@@ -163,8 +163,8 @@
     return TT_Err_Ok;
 
   Fail1:
-    for ( n = 0; n < count; n++ )
-      Free_LangSys( &lsr[n].LangSys, memory );
+    for ( m = 0; m < n; m++ )
+      Free_LangSys( &lsr[m].LangSys, memory );
 
     FREE( s->LangSysRecord );
 
@@ -205,7 +205,7 @@
     FT_Error   error;
     FT_Memory  memory = stream->memory;
 
-    FT_UShort          n, count;
+    FT_UShort          n, m, count;
     FT_ULong           cur_offset, new_offset, base_offset;
 
     TTO_ScriptRecord*  sr;
@@ -247,8 +247,8 @@
     return TT_Err_Ok;
 
   Fail:
-    for ( n = 0; n < count; n++ )
-      Free_Script( &sr[n].Script, memory );
+    for ( m = 0; m < n; m++ )
+      Free_Script( &sr[m].Script, memory );
 
     FREE( sl->ScriptRecord );
     return error;
@@ -340,7 +340,7 @@
     FT_Error   error;
     FT_Memory  memory = stream->memory;
 
-    FT_UShort           n, count;
+    FT_UShort           n, m, count;
     FT_ULong            cur_offset, new_offset, base_offset;
 
     TTO_FeatureRecord*  fr;
@@ -382,8 +382,8 @@
     return TT_Err_Ok;
 
   Fail:
-    for ( n = 0; n < count; n++ )
-      Free_Feature( &fr[n].Feature, memory );
+    for ( m = 0; m < n; m++ )
+      Free_Feature( &fr[m].Feature, memory );
 
     FREE( fl->FeatureRecord );
     return error;
@@ -454,7 +454,6 @@
     else
       switch ( lookup_type )
       {
-#if 0
         case GPOS_LOOKUP_SINGLE:
           return Load_SinglePos( &st->st.gpos.single, stream );
 
@@ -479,7 +478,6 @@
         case GPOS_LOOKUP_CHAIN:
           return Load_ChainContextPos( &st->st.gpos.chain, stream );
 
-#endif
         default:
           return TTO_Err_Invalid_GPOS_SubTable_Format;
       }
@@ -523,39 +521,37 @@
     else
       switch ( lookup_type )
       {
-#if 0
       case GPOS_LOOKUP_SINGLE:
-        Free_SinglePos( &st->st.gpos.single );
+        Free_SinglePos( &st->st.gpos.single, memory );
         break;
 
       case GPOS_LOOKUP_PAIR:
-        Free_PairPos( &st->st.gpos.pair );
+        Free_PairPos( &st->st.gpos.pair, memory );
         break;
 
       case GPOS_LOOKUP_CURSIVE:
-        Free_CursivePos( &st->st.gpos.cursive );
+        Free_CursivePos( &st->st.gpos.cursive, memory );
         break;
 
       case GPOS_LOOKUP_MARKBASE:
-        Free_MarkBasePos( &st->st.gpos.markbase );
+        Free_MarkBasePos( &st->st.gpos.markbase, memory );
         break;
 
       case GPOS_LOOKUP_MARKLIG:
-        Free_MarkLigPos( &st->st.gpos.marklig );
+        Free_MarkLigPos( &st->st.gpos.marklig, memory );
         break;
 
       case GPOS_LOOKUP_MARKMARK:
-        Free_MarkMarkPos( &st->st.gpos.markmark );
+        Free_MarkMarkPos( &st->st.gpos.markmark, memory );
         break;
 
       case GPOS_LOOKUP_CONTEXT:
-        Free_ContextPos( &st->st.gpos.context );
+        Free_ContextPos( &st->st.gpos.context, memory );
         break;
 
       case GPOS_LOOKUP_CHAIN:
-        Free_ChainContextPos ( &st->st.gpos.chain );
+        Free_ChainContextPos ( &st->st.gpos.chain, memory );
         break;
-#endif
       }
   }
 
@@ -569,7 +565,7 @@
     FT_Error   error;
     FT_Memory  memory = stream->memory;
 
-    FT_UShort      n, count;
+    FT_UShort      n, m, count;
     FT_ULong       cur_offset, new_offset, base_offset;
 
     TTO_SubTable*  st;
@@ -613,8 +609,8 @@
     return TT_Err_Ok;
 
   Fail:
-    for ( n = 0; n < count; n++ )
-      Free_SubTable( &st[n], type, l->LookupType, memory );
+    for ( m = 0; m < n; m++ )
+      Free_SubTable( &st[m], type, l->LookupType, memory );
 
     FREE( l->SubTable );
     return error;
@@ -652,7 +648,7 @@
     FT_Error   error;
     FT_Memory  memory = stream->memory;
 
-    FT_UShort    n, count;
+    FT_UShort    n, m, count;
     FT_ULong     cur_offset, new_offset, base_offset;
 
     TTO_Lookup*  l;
@@ -697,8 +693,8 @@
   Fail1:
     FREE( ll->Properties );
 
-    for ( n = 0; n < count; n++ )
-      Free_Lookup( &l[n], type, memory );
+    for ( m = 0; m < n; m++ )
+      Free_Lookup( &l[m], type, memory );
 
   Fail2:
     FREE( ll->Lookup );
@@ -1204,6 +1200,29 @@
   }
 
 
+  FT_Error  Load_EmptyClassDefinition( TTO_ClassDefinition*  cd,
+				       FT_Stream             stream )
+  {
+    FT_Error   error;
+    FT_Memory  memory = stream->memory;
+
+
+    if ( ALLOC_ARRAY( cd->Defined, 1, FT_Bool ) )
+      return error;
+
+    cd->ClassFormat = 1; /* Meaningless */
+    cd->Defined[0] = FALSE;
+
+    if ( ALLOC_ARRAY( cd->cd.cd1.ClassValueArray, 1, FT_UShort ) )
+      goto Fail;
+
+    return TT_Err_Ok;
+
+  Fail:
+    FREE( cd->Defined );
+    return error;
+  }
+
   void  Free_ClassDefinition( TTO_ClassDefinition*  cd,
 			      FT_Memory             memory )
   {
@@ -1440,7 +1459,7 @@
 
     f = d->DeltaFormat;
 
-    if ( size >= d->StartSize && size <= d->EndSize )
+    if ( d->DeltaValue && size >= d->StartSize && size <= d->EndSize )
     {
       s    = size - d->StartSize;
       byte = d->DeltaValue[s >> ( 4 - f )];
