@@ -33,7 +33,6 @@
 #include "pango-utils.h"
 #include "pangoft2.h"
 #include "pangoft2-private.h"
-#include "modules.h"
 
 #define PANGO_TYPE_FT2_FONT              (pango_ft2_font_get_type ())
 #define PANGO_FT2_FONT(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), PANGO_TYPE_FT2_FONT, PangoFT2Font))
@@ -95,7 +94,7 @@ static GType pango_ft2_font_get_type (void);
 
 PangoFT2Font *
 _pango_ft2_font_new (PangoFontMap    *fontmap,
-		     MiniXftPattern  *pattern)
+		     FcPattern  *pattern)
 {
   PangoFT2Font *ft2font;
   double d;
@@ -112,7 +111,7 @@ _pango_ft2_font_new (PangoFontMap    *fontmap,
   ft2font->description = _pango_ft2_font_desc_from_pattern (pattern, TRUE);
   ft2font->face = NULL;
 
-  if (MiniXftPatternGetDouble (pattern, XFT_PIXEL_SIZE, 0, &d) == MiniXftResultMatch)
+  if (FcPatternGetDouble (pattern, FC_PIXEL_SIZE, 0, &d) == FcResultMatch)
     ft2font->size = d*PANGO_SCALE;
 
   _pango_ft2_font_map_add (ft2font->fontmap, ft2font);
@@ -120,6 +119,73 @@ _pango_ft2_font_new (PangoFontMap    *fontmap,
   return ft2font;
 }
 
+static gboolean
+set_unicode_charmap (FT_Face face)
+{
+  int charmap;
+
+  for (charmap = 0; charmap < face->num_charmaps; charmap++)
+    if (face->charmaps[charmap]->encoding == ft_encoding_unicode)
+      {
+	FT_Error error = FT_Set_Charmap(face, face->charmaps[charmap]);
+	return error == FT_Err_Ok;
+      }
+
+  return FALSE;
+}
+
+static void
+load_fallback_face (PangoFT2Font *ft2font,
+		    const char   *original_file)
+{
+  FcPattern *sans;
+  FcPattern *matched;
+  FcResult result;
+  FT_Error error;
+  FcChar8 *filename2 = NULL;
+  gchar *name;
+  int id;
+  
+  sans = FcPatternBuild (NULL,
+			 FC_FAMILY, FcTypeString, "sans",
+			 FC_SIZE, FcTypeDouble, (double)pango_font_description_get_size (ft2font->description)/PANGO_SCALE,
+			 NULL);
+  
+  matched = FcFontMatch (0, sans, &result);
+  
+  if (FcPatternGetString (matched, FC_FILE, 0, &filename2) != FcResultMatch)
+    goto bail1;
+  
+  if (FcPatternGetInteger (matched, FC_INDEX, 0, &id) != FcResultMatch)
+    goto bail1;
+  
+  error = FT_New_Face (_pango_ft2_font_map_get_library (ft2font->fontmap),
+		       (char *) filename2, id, &ft2font->face);
+  
+  
+  if (error)
+    {
+    bail1:
+      name = pango_font_description_to_string (ft2font->description);
+      g_warning ("Unable to open font file %s for font %s, exiting\n", filename2, name);
+      exit (1);
+    }
+  else
+    {
+      name = pango_font_description_to_string (ft2font->description);
+      g_warning ("Unable to open font file %s for font %s, falling back to %s\n", original_file, name, filename2);
+      g_free (name);
+    }
+
+  if (!set_unicode_charmap (ft2font->face))
+    {
+      g_warning ("Unable to load unicode charmap from file %s, exiting\n", filename2);
+      exit (1);
+    }
+  
+  FcPatternDestroy (sans);
+  FcPatternDestroy (matched);
+}
 
 /**
  * pango_ft2_font_get_face:
@@ -136,72 +202,62 @@ pango_ft2_font_get_face (PangoFont      *font)
   PangoFT2Font *ft2font = (PangoFT2Font *)font;
   FT_Face face;
   FT_Error error;
-  MiniXftPattern *pattern;
-  char *filename;
+  FcPattern *pattern;
+  FcChar8 *filename;
+  FcBool hinting, autohint;
   int id;
 
   pattern = ft2font->font_pattern;
 
   if (!ft2font->face)
     {
-      if (MiniXftPatternGetString (pattern, XFT_FILE, 0, &filename) != MiniXftResultMatch)
-	goto bail0;
-    
-      if (MiniXftPatternGetInteger (pattern, XFT_INDEX, 0, &id) != MiniXftResultMatch)
-	goto bail0;
-    
+      /* If we add support for not antialising, then we should
+       * need to conditionalize NO_BITMAP on that.
+       */
+      ft2font->load_flags = FT_LOAD_NO_BITMAP;
+
+      /* disable hinting if requested */
+      if (FcPatternGetBool (pattern, FC_HINTING, 0, &hinting) != FcResultMatch)
+	hinting = FcTrue;
+
+      if (!hinting)
+        ft2font->load_flags |= FT_LOAD_NO_HINTING;
+
+      /* force autohinting if requested */
+      if (FcPatternGetBool (pattern, FC_AUTOHINT, 0, &autohint) != FcResultMatch)
+	autohint = FcFalse;
+
+      if (autohint)
+        ft2font->load_flags |= FT_LOAD_FORCE_AUTOHINT;
+
+      if (FcPatternGetString (pattern, FC_FILE, 0, &filename) != FcResultMatch)
+	      goto bail0;
+      
+      if (FcPatternGetInteger (pattern, FC_INDEX, 0, &id) != FcResultMatch)
+	      goto bail0;
+
       error = FT_New_Face (_pango_ft2_font_map_get_library (ft2font->fontmap),
-			   filename, id, &ft2font->face);
-      if (error)
+			   (char *) filename, id, &ft2font->face);
+      if (error != FT_Err_Ok)
 	{
-	  MiniXftPattern *sans;
-	  MiniXftPattern *matched;
-	  MiniXftResult result;
-	  char *filename2 = NULL;
-	  gchar *name;
-
 	bail0:
-	  
-	  sans = MiniXftPatternBuild (0,
-				      XFT_FAMILY, MiniXftTypeString, "sans",
-				      XFT_ENCODING, MiniXftTypeString, "glyphs-fontspecific",
-				      XFT_SIZE, MiniXftTypeDouble, (double)pango_font_description_get_size (ft2font->description)/PANGO_SCALE,
-				      NULL);
-
-	  matched = MiniXftFontMatch ((Display *)1, 0, sans, &result);
-
-	  if (MiniXftPatternGetString (matched, XFT_FILE, 0, &filename2) != MiniXftResultMatch)
-	    goto bail1;
-	  
-	  if (MiniXftPatternGetInteger (matched, XFT_INDEX, 0, &id) != MiniXftResultMatch)
-	    goto bail1;
-	  
-	  error = FT_New_Face (_pango_ft2_font_map_get_library (ft2font->fontmap),
-			       filename2, id, &ft2font->face);
-
-	  
-	  if (error)
-	    {
-	    bail1:
-	      name = pango_font_description_to_string (ft2font->description);
-	      g_warning ("Unable to open font file %s for font %s, exiting\n", filename2, name);
-	      exit (1);
-	    }
-	  else
-	    {
-	      name = pango_font_description_to_string (ft2font->description);
-	      g_warning ("Unable to open font file %s for font %s, falling back to %s\n", filename, name, filename2);
-	      g_free (name);
-	    }
-
-	  MiniXftPatternDestroy (sans);
-	  MiniXftPatternDestroy (matched);
+	  load_fallback_face (ft2font, filename);
 	}
       ft2font->face->generic.data = 0;
     }
 
   g_assert (ft2font->face);
   
+  if (!set_unicode_charmap (ft2font->face))
+    {
+      g_warning ("Unable to load unicode charmap from font file %s", filename);
+      
+      FT_Done_Face (ft2font->face);
+      ft2font->face = NULL;
+
+      load_fallback_face (ft2font, filename);
+    }
+
   face = ft2font->face;
 
   if (ft2font->size != GPOINTER_TO_UINT (face->generic.data))
@@ -219,39 +275,6 @@ pango_ft2_font_get_face (PangoFont      *font)
   return face;
 }
 
-/**
- * pango_ft2_get_context:
- * @dpi_x:  the horizontal dpi of the target device
- * @dpi_y:  the vertical dpi of the target device
- * 
- * Retrieves a #PangoContext appropriate for rendering with the PangoFT2
- * backend.
- * 
- * Return value: the new #PangoContext
- **/
-PangoContext *
-pango_ft2_get_context (double dpi_x, double dpi_y)
-{
-  PangoContext *result;
-  static gboolean registered_modules = FALSE;
-  int i;
-  
-  if (!registered_modules)
-    {
-      registered_modules = TRUE;
-      
-      for (i = 0; _pango_included_ft2_modules[i].list; i++)
-        pango_module_register (&_pango_included_ft2_modules[i]);
-    }
-
-  MiniXftSetDPI (dpi_y);
-  
-  result = pango_context_new ();
-  pango_context_set_font_map (result, pango_ft2_font_map_for_display ());
-
-  return result;
-}
-
 static GType
 pango_ft2_font_get_type (void)
 {
@@ -330,9 +353,10 @@ pango_ft2_font_render_glyph (PangoFont *font,
   
   if (face)
     {
+      PangoFT2Font *ft2font = (PangoFT2Font *)font;
+	    
       /* Draw glyph */
-      /* FIXME hint or not? */ 
-      FT_Load_Glyph (face, glyph_index, FT_LOAD_DEFAULT);
+      FT_Load_Glyph (face, glyph_index, ft2font->load_flags);
       FT_Render_Glyph (face->glyph, ft_render_mode_normal);
 
       rendered->bitmap = face->glyph->bitmap;
@@ -473,10 +497,15 @@ pango_ft2_render (FT_Bitmap        *bitmap,
 }
 
 static FT_Glyph_Metrics *
-pango_ft2_get_per_char (FT_Face   face,
-			guint32   glyph_index)
+pango_ft2_get_per_char (PangoFont *font,
+			guint32    glyph_index)
 {
-  FT_Load_Glyph (face, glyph_index, FT_LOAD_DEFAULT);
+  PangoFT2Font *ft2font = (PangoFT2Font *)font;
+  FT_Face face;
+	
+  face = pango_ft2_font_get_face (font);
+  
+  FT_Load_Glyph (face, glyph_index, ft2font->load_flags);
   return &face->glyph->metrics;
 }
 
@@ -496,7 +525,7 @@ pango_ft2_font_get_glyph_info (PangoFont   *font,
       FT_Face face = pango_ft2_font_get_face (font);
       info = g_new0 (PangoFT2GlyphInfo, 1);
       
-      if (glyph && (gm = pango_ft2_get_per_char (face, glyph)))
+      if (glyph && (gm = pango_ft2_get_per_char (font, glyph)))
 	{
 	  info->ink_rect.x = PANGO_UNITS_26_6 (gm->horiBearingX);
 	  info->ink_rect.width = PANGO_UNITS_26_6 (gm->width);
@@ -648,46 +677,6 @@ pango_ft2_font_get_metrics (PangoFont     *font,
   return pango_font_metrics_ref (info->metrics);
 }
 
-static PangoCoverage *
-pango_ft2_calc_coverage (PangoFont     *font,
-			 PangoLanguage *language)
-{
-  PangoCoverage *coverage;
-  FT_Face face;
-
-  coverage = pango_coverage_new ();
-  face = pango_ft2_font_get_face (font);
-
-#ifdef HAVE_FT_GET_FIRST_CHAR
-  {
-    FT_UInt gindex;
-    FT_ULong charcode;
-    
-    charcode = FT_Get_First_Char (face, &gindex);
-    while (gindex)
-      {
-	pango_coverage_set (coverage, charcode, PANGO_COVERAGE_EXACT);
-	charcode = FT_Get_Next_Char (face, charcode, &gindex);
-      }
-  }
-#else  
-  /* Ugh, this is going to be SLOW */
-  {
-    gunichar wc;
-    
-    for (wc = 0; wc < G_MAXUSHORT; wc++)
-      {
-	FT_UInt glyph = FT_Get_Char_Index (face, wc);
-	
-	if (glyph && glyph < face->num_glyphs)
-	  pango_coverage_set (coverage, wc, PANGO_COVERAGE_EXACT);
-      }
-  }
-#endif
-
-  return coverage;
-}
-
 static void
 pango_ft2_font_dispose (GObject *object)
 {
@@ -738,7 +727,7 @@ pango_ft2_font_finalize (GObject *object)
     }
 
   pango_font_description_free (ft2font->description);
-  MiniXftPatternDestroy (ft2font->font_pattern);
+  FcPatternDestroy (ft2font->font_pattern);
   
   g_object_unref (G_OBJECT (ft2font->fontmap));
 
@@ -793,26 +782,8 @@ pango_ft2_font_get_coverage (PangoFont     *font,
 			     PangoLanguage *language)
 {
   PangoFT2Font *ft2font = (PangoFT2Font *)font;
-  char *filename = NULL;
-  FT_Face face;
-  PangoCoverage *coverage;
-  
-  MiniXftPatternGetString (ft2font->font_pattern, XFT_FILE, 0, &filename);
-  
-  coverage = _pango_ft2_font_map_get_coverage (ft2font->fontmap, filename);
 
-  if (coverage)
-    return pango_coverage_ref (coverage);
-
-  /* Ugh, this is going to be SLOW */
-
-  face = pango_ft2_font_get_face (font);
-
-  coverage = pango_ft2_calc_coverage (font, language);
-
-  _pango_ft2_font_map_set_coverage (ft2font->fontmap, filename, coverage);
-      
-  return coverage;
+  return _pango_ft2_font_map_get_coverage (ft2font->fontmap, ft2font->font_pattern);
 }
 
 static PangoEngineShape *
